/*
    Crystal Space 3D engine
    Copyright (C) 2002 by Erik Nihlmar and Mikael Rosberg
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __IENGINE_VEGFUNC_H__
#define __IENGINE_VEGFUNC_H__

#include "csutil/scf.h"
#include "imap/parser.h"
#include "ivideo/graph3d.h"
#include "iengine/engine.h"
#include "cstool/csview.h"
#include "imesh/terrfunc.h"

SCF_VERSION (iVegFuncSystem, 0, 0, 1);

/**
 * This is the vegetation planter system plugin.
 * It is meant to be used with the terrain plugin,
 * to ease the plantation of for example an forrest.
 */

struct iVegFuncSystem : public iBase
{
  /**
   * This is just an implementation of an dum contact method. No returns.
   */
  virtual void Contact() = 0;

  virtual void Init_VegFunc(iLoader* loader, iGraphics3D* g3d,  iView *view) = 0;

  virtual void Init_VegFunc_Height(iTerrainHeightFunction* height, iTerrFuncState* terr_state, iSector* room) = 0;

  virtual void Load_objects(boolean blend,float max_bl_dist, float min_bl_dist, boolean rotation, float rot_dist,
							csVector2* upper_left_vertex, csVector2* lower_right_vertex, int nmb_of_species, 
						   float min_dist,char* name, char* path,char* name_i = NULL, char* path_i = NULL,
						   char* name_ii = NULL, char* path_ii = NULL) = 0;

  virtual void Load_object(boolean blend, float max_bl_dist, float min_bl_dist, boolean rotation, float rot_dist,
								 float x, float z,char* name, char* path, char* name_i = NULL, char* path_i = NULL,
								 char* name_ii = NULL, char* path_ii = NULL) = 0;

  virtual void Place_Object(float x, float z, char* path_i, char* path_ii = NULL, char* path_iii = NULL ) = 0;

  virtual void Place_Objects() = 0;

  virtual void Blending() = 0;

  virtual void Billboard_Rotation () = 0;
};

#endif // __IENGINE_VEGFUNC_H__

