/*
Copyright (C) 2002 by Erik Nihlmar and Mikael Rosberg

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
	
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "cssysdef.h"
#include "csvegfunc.h"

#include "iengine/mesh.h"
#include "iengine/sector.h"
#include "iengine/movable.h"
#include "iengine/camera.h"
#include "imap/parser.h"
#include "ivideo/graph3d.h"
#include "ivideo/txtmgr.h"
#include "imesh/sprite3d.h"
#include "imesh/object.h"
#include "cstool/csview.h"
#include "csgeom/math3d.h"


#define HEIGHT_OFFSET 0.3
#define MAX_BLEND_DIST 1000.0
#define MIN_BLEND_DIST 800.0
#define ROT_DIST 400.0

CS_IMPLEMENT_PLUGIN

//----------------------------------------------------------------------

SCF_IMPLEMENT_IBASE (csVegFuncSystem)
SCF_IMPLEMENTS_INTERFACE (iVegFuncSystem)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE (iComponent)
SCF_IMPLEMENT_IBASE_END


SCF_IMPLEMENT_EMBEDDED_IBASE (csVegFuncSystem::eiComponent)
SCF_IMPLEMENTS_INTERFACE (iComponent)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_FACTORY (csVegFuncSystem)

SCF_EXPORT_CLASS_TABLE (vegfunc)
SCF_EXPORT_CLASS (csVegFuncSystem,"crystalspace.vegetationplanter.vegfunc",
				  "Crystal Space Vegetation Planter System")
				  SCF_EXPORT_CLASS_TABLE_END
				  
				  
				  
				  
				  
csVegFuncSystem::csVegFuncSystem (iBase *pParent)
{
	SCF_CONSTRUCT_IBASE (pParent);
	SCF_CONSTRUCT_EMBEDDED_IBASE(scfiComponent);
	
	loader = NULL;
	view = NULL;
	g3d = NULL;
	room = NULL;
	
	
	for(int i=0;i<NUMBER_OF_OBJECTS;i++){
		veg_vec[i].lrv = NULL;
		veg_vec[i].ulv = NULL;
		veg_vec[i].origin = NULL;
		veg_vec[i].bill_name = NULL;
		veg_vec[i].mobj_i_name = NULL;
		veg_vec[i].mobj_ii_name = NULL;
		veg_vec[i].wrapp_bill = NULL;
		veg_vec[i].wrapp_mobj_i = NULL;
		veg_vec[i].wrapp_mobj_ii = NULL;
		veg_vec[i].md = 0.0f;
		veg_vec[i].nmbos = 1;
		veg_vec[i].blending = FALSE;
		//---------------HACK----------
		veg_vec[i].switching = FALSE;
		//_----------------------------
		veg_vec[i].names = NULL;
		veg_vec[i].max_blend_dist = NULL;
		veg_vec[i].min_blend_dist =  NULL;
		veg_vec[i].x = 0.0f;
		veg_vec[i].z = 0.0f;
	}
}

csVegFuncSystem::~csVegFuncSystem(){
	
}


struct random_data {
	float x;
	float z;
	float alpha;
	int nmbofs;
};
unsigned int curr = 0;
boolean first = true;
random_data* Random(csVector2* ulv, csVector2* lrv,int nmbofs, float md) {
	
	random_data* veg_data= new random_data[nmbofs];
	
	
	float x0 = ulv->x, z0 = ulv->y;
	
	float xd = (lrv->x) - (ulv->x);
	float zd = (lrv->y) - (ulv->y);
	
	float max_obj = (abs(xd*zd)/((1.0f+2.0f*md)*(1.0f+2.0f*md)));
	
	if((float) nmbofs > max_obj){
		nmbofs = (int) max_obj;
	}
	veg_data[0].nmbofs = nmbofs;
	printf("nmbof: %d, max_obj: %f\n",nmbofs,max_obj);
	
	//srand(11781);
	if(first){
		curr = (unsigned)time( NULL );
		first = false;
		srand( curr );
		printf("CURR: %d\n",curr);
	}
	else{
		curr = curr + 100;
		srand( curr );
		printf("CURR: %d\n",curr);
	}

	float xc = 0.0f, zc = 0.0f, alpha = 0.0f;
	
	float f_max = (float) RAND_MAX;
	
	
	veg_data[0].x = x0 + (((float)rand()/f_max)*xd);
	veg_data[0].z = z0 + (((float)rand()/f_max)*zd);
    veg_data[0].alpha = (((float)rand()/f_max)*2*PI);
	
	for(int i=1;i<nmbofs;i++){
		
		//--------- x , z -----------------
		xc = x0 + (((float) rand()/f_max)*xd);
		zc = z0 + (((float) rand()/f_max)*zd);
		for(int j=0;j<i;j++){
			if(abs(veg_data[j].x - xc) < md && abs(veg_data[j].z - zc) < md){
				while(abs(veg_data[j].x - xc) < md && abs(veg_data[j].z - zc) < md){
					xc = x0 + (((float) rand()/f_max)*xd);
					zc = z0 + (((float) rand()/f_max)*zd);
				}
				j = 0;
			}
		}
		veg_data[i].x = xc;
		veg_data[i].z = zc;
		
		//---------- alpha ---------------
		veg_data[i].alpha = (((float)rand()/f_max)*2*PI);
	}	
		  
	return veg_data;
	
}

void add_sprite(iView* view, iSector* room, iMeshFactoryWrapper* sprite_tmpl, float x, float y, float z, 
				const char* name="MySprite", bool b = TRUE, float r = 0.0f){
	
	iMeshWrapper* sprite = view->GetEngine()->CreateMeshWrapper(sprite_tmpl, name, room, csVector3 (x,y,z)); 
	//csMatrix3 m; m.Identity (); m *= .5;
	//sprite->GetMovable ()->SetTransform (m);
	
	
	if(!b)
		sprite->GetMovable()->ClearSectors();
	
	iSprite3DState* spstate = SCF_QUERY_INTERFACE (sprite->GetMeshObject (),iSprite3DState);
	spstate->SetBaseColor(csColor(1,1,1));
	spstate->SetAction ("default");
	spstate->DecRef ();
	
	sprite->SetZBufMode (CS_ZBUF_TEST);
	sprite->SetRenderPriority (view->GetEngine()->GetRenderPriority ("tree"));
	sprite->GetMovable()->SetTransform(csYRotMatrix3((r)));
	sprite->GetMovable()->UpdateMove();
} 


void rotate_sprite(iMeshWrapper* billboard_sprite, const csVector3& origin, iView* view, iMeshWrapper* polygon_sprite_1=NULL, iMeshWrapper* polygon_sprite_2=NULL){
	
	const csVector3& camera_origin = origin;
	const csVector3& billboard_origin = billboard_sprite->GetMovable()->GetPosition();
	
	float delta_x = camera_origin.x - billboard_origin.x;
	float delta_z = camera_origin.z - billboard_origin.z;
	
	float alpha_tmp = 0;
	
	if(delta_z != 0)
		alpha_tmp = atan2f(delta_x,delta_z);
	
	billboard_sprite->GetMovable()->SetTransform(csYRotMatrix3(-alpha_tmp));
	billboard_sprite->GetMovable()->UpdateMove();
	
	if(polygon_sprite_1!=NULL){
		polygon_sprite_1->GetMovable()->SetTransform(csYRotMatrix3(-alpha_tmp));
		polygon_sprite_1->GetMovable()->UpdateMove();
	}
	
	if(polygon_sprite_2!=NULL){
		polygon_sprite_2->GetMovable()->SetTransform(csYRotMatrix3(-alpha_tmp));
		polygon_sprite_2->GetMovable()->UpdateMove();
	}
}


//------------ A simple method, just checking if we are alive ------------
void csVegFuncSystem::Contact(){
	printf("HOUSTON, WE GOT CONTACT !!!!!\n");	
	/*for(int i=0;i < 20;i++){
	ostringstream ost1;
	char* ch = "char";
	string s = ch;
	ost1 << s << i;
	string str = ost1.str();		
	printf("%s \n",str.c_str());
}*/
}


void csVegFuncSystem::Init_VegFunc(iLoader* loader, iGraphics3D* g3d, iView* view){	
	
	this->loader = loader;
	this->g3d = g3d;
	this->view = view;
	
	veg_vec_ind = 0;
	
}

void csVegFuncSystem::Init_VegFunc_Height(iTerrainHeightFunction* height, iTerrFuncState* terr_state, iSector* room){
	
	this->room = room;
	this->height = height;
	this->terr_state = terr_state;
	
}




void csVegFuncSystem::Billboard_Rotation(){
	
	
	for(int i=0;i < veg_vec_ind; i++){
		
		if(veg_vec[i].rotation){
			
			for(int j=0; j < veg_vec[i].nmbos;j++){	
				iMeshWrapper* sprite_bill = view->GetEngine()->FindMeshObject(veg_vec[i].names[j].bill_name->GetData());				
				
				iCamera* camera = view->GetCamera ();
				csReversibleTransform& camtrans = camera->GetTransform ();
				const csVector3& origin = camtrans.GetOrigin ();
				const csVector3& bill_origin = sprite_bill->GetMovable()->GetPosition();
	
				float sqdist = csSquaredDist::PointPoint (bill_origin, origin);
				
				if(sqdist<veg_vec[i].rot_dist){

					csTransform bill_trans = sprite_bill->GetMovable()->GetTransform();
					csMatrix3 bill_matr = bill_trans.GetO2T();
		
					csTransform cam_trans = view->GetCamera()->GetTransform();
					csMatrix3 cam_matr = cam_trans.GetO2T();
		
					float delta_x = origin.x - bill_origin.x;
					float delta_z = origin.z - bill_origin.z;
		
					float alpha_tmp = 0;
		
					if(delta_z != 0)
						alpha_tmp = atan2f(delta_x,delta_z);
		
					sprite_bill->GetMovable()->SetTransform(csYRotMatrix3((-alpha_tmp)));
					sprite_bill->GetMovable()->UpdateMove();
				}
			}
		}
	}
}



void csVegFuncSystem::Blending(){
	
	iCamera* camera = view->GetCamera ();
	csReversibleTransform& camtrans = camera->GetTransform ();
	const csVector3& origin = camtrans.GetOrigin ();
	
	for(int i=0;i < veg_vec_ind; i++){
		
		if(veg_vec[i].blending){
			
			for(int j=0; j <veg_vec[i].nmbos;j++){
				
				
				
				//name_data* names = veg_vec[i].names;
				
				//printf("I: %d\n",i);
				//printf("NAME: %s \n",veg_vec[i].names[j].bill_name->GetData());
				
				
				iMeshWrapper* sprite_poly_1 = view->GetEngine()->FindMeshObject(veg_vec[i].names[j].mobj_i_name->GetData());
				iMeshWrapper* sprite_poly_2 = view->GetEngine()->FindMeshObject(veg_vec[i].names[j].mobj_ii_name->GetData());
				iMeshWrapper* sprite_bill = view->GetEngine()->FindMeshObject(veg_vec[i].names[j].bill_name->GetData());
				
				float bill_alpha = veg_vec[i].names[j].bill_alpha;
				float poly_alpha = veg_vec[i].names[j].poly_alpha;
				
				//printf("DISTANCE: %f\n",sqdist);
				
				uint bill_Mixmode = 0;
				uint poly_Mixmode = 0;
				
				const csVector3& obj_vec = sprite_poly_1->GetMovable()->GetPosition();
				
				float sqdist = csSquaredDist::PointPoint (obj_vec, origin);
				
				//printf("MAX_BLEND_DIST: %f \n",veg_vec[i].max_blend_dist);

				if((sqdist > veg_vec[i].max_blend_dist)  ){//&& (sqdist < 10000)
					
					bill_Mixmode &= ~CS_FX_MASK_ALPHA;
					poly_Mixmode &= ~CS_FX_MASK_ALPHA;
					
					if(bill_alpha > 0.0f)
						bill_alpha = bill_alpha - 0.1f;
					if(poly_alpha < 1.0f)
						poly_alpha = poly_alpha + 0.1f;
					
					bill_Mixmode |= CS_FX_SETALPHA (bill_alpha);
					poly_Mixmode |= CS_FX_SETALPHA (poly_alpha);
					
					
					if(bill_alpha < 1.0f){ 
						sprite_bill->GetMovable()->SetSector(room);
						rotate_sprite(sprite_bill,origin,view,sprite_poly_1,sprite_poly_2);
					}
					if(bill_alpha < 0.0f){
						sprite_poly_1->GetMovable()->ClearSectors();
						sprite_poly_2->GetMovable()->ClearSectors();
						rotate_sprite(sprite_bill,origin,view);
					}		
				}
				/*if(sqdist > 10000){
					sprite_bill->GetMovable()->ClearSectors();
					sprite_poly_1->GetMovable()->ClearSectors();
					sprite_poly_2->GetMovable()->ClearSectors();
				}*/
				
				
				if(sqdist < veg_vec[i].min_blend_dist){
					
				/*uint bill_Mixmode = 0;
					uint poly_Mixmode = 0;*/
					
					bill_Mixmode &= ~CS_FX_MASK_ALPHA;
					poly_Mixmode &= ~CS_FX_MASK_ALPHA;
					
					if(poly_alpha > 0.0f)
						poly_alpha = poly_alpha - 0.1f;
					if(bill_alpha < 1.0f)
						bill_alpha = bill_alpha + 0.1f;
					
					bill_Mixmode |= CS_FX_SETALPHA (bill_alpha);
					poly_Mixmode |= CS_FX_SETALPHA (poly_alpha);
					
					
					if(poly_alpha < 1.0f) {			
						sprite_poly_1->GetMovable()->SetSector(room);
						sprite_poly_2->GetMovable()->SetSector(room);
					}
					if(poly_alpha < 0.0f)
						sprite_bill->GetMovable()->ClearSectors();
					
					
					
				}
				
				iSprite3DState* poly_obj_1 = SCF_QUERY_INTERFACE (sprite_poly_1->GetMeshObject (),iSprite3DState);
				poly_obj_1->SetBaseColor(csColor(1,1,1));
				poly_obj_1->SetMixMode(poly_Mixmode);
				poly_obj_1->SetAction ("default");
				poly_obj_1->DecRef ();
				
				iSprite3DState* poly_obj_2 = SCF_QUERY_INTERFACE (sprite_poly_2->GetMeshObject (),iSprite3DState);
				poly_obj_2->SetBaseColor(csColor(1,1,1));
				poly_obj_2->SetMixMode(poly_Mixmode);
				poly_obj_2->SetAction ("default");
				poly_obj_2->DecRef ();
				
				iSprite3DState* bill_obj = SCF_QUERY_INTERFACE (sprite_bill->GetMeshObject (),iSprite3DState);
				bill_obj->SetBaseColor(csColor(1,1,1));
				bill_obj->SetMixMode(bill_Mixmode);
				bill_obj->SetAction ("default");
				bill_obj->DecRef ();
				
				veg_vec[i].names[j].bill_alpha = bill_alpha;
				veg_vec[i].names[j].poly_alpha = poly_alpha;
			}
		}	

		if(veg_vec[i].switching){
			
			iMeshWrapper* sprite_poly_1 = view->GetEngine()->FindMeshObject(veg_vec[i].names[0].mobj_i_name->GetData());
			iMeshWrapper* sprite_poly_2 = view->GetEngine()->FindMeshObject(veg_vec[i].names[0].mobj_ii_name->GetData());
			iMeshWrapper* sprite_bill = view->GetEngine()->FindMeshObject(veg_vec[i].names[0].bill_name->GetData());

			const csVector3& obj_vec = sprite_poly_1->GetMovable()->GetPosition();
				
			float sqdist = csSquaredDist::PointPoint (obj_vec, origin);

			if(sqdist > veg_vec[i].max_blend_dist){
				sprite_bill->GetMovable()->SetSector(room);
				sprite_poly_1->GetMovable()->ClearSectors();
				sprite_poly_2->GetMovable()->ClearSectors();
				rotate_sprite(sprite_bill,origin,view,sprite_poly_1,sprite_poly_2);
			
			}
			
			if(sqdist < veg_vec[i].min_blend_dist){
				sprite_poly_1->GetMovable()->SetSector(room);
				sprite_poly_2->GetMovable()->SetSector(room);
				sprite_bill->GetMovable()->ClearSectors();
				rotate_sprite(sprite_bill,origin,view,sprite_poly_1,sprite_poly_2);
			}

			iSprite3DState* poly_obj_1 = SCF_QUERY_INTERFACE (sprite_poly_1->GetMeshObject (),iSprite3DState);
			poly_obj_1->SetBaseColor(csColor(1,1,1));
			
			poly_obj_1->SetAction ("default");
			poly_obj_1->DecRef ();
				
			iSprite3DState* poly_obj_2 = SCF_QUERY_INTERFACE (sprite_poly_2->GetMeshObject (),iSprite3DState);
			poly_obj_2->SetBaseColor(csColor(1,1,1));
			
			poly_obj_2->SetAction ("default");
			poly_obj_2->DecRef ();
				
			iSprite3DState* bill_obj = SCF_QUERY_INTERFACE (sprite_bill->GetMeshObject (),iSprite3DState);
			bill_obj->SetBaseColor(csColor(1,1,1));
			
			bill_obj->SetAction ("default");
			bill_obj->DecRef ();
				
		}
	}
}





void csVegFuncSystem::Load_objects(boolean blend, float max_bl_dist, float min_bl_dist, boolean rotation, float rot_dist,
								   csVector2* upper_left_vertex, csVector2* lower_right_vertex,
								   int nmb_of_species, float min_dist,char* name, char* path, 
								   char* name_i = NULL, char* path_i = NULL,char* name_ii= NULL, 
								   char* path_ii = NULL){
	
	
	if(path_ii == NULL){
		if(path_i == NULL){
			veg_vec[veg_vec_ind].wrapp_bill = loader->LoadMeshObjectFactory(path);
			veg_vec[veg_vec_ind].bill_name = name;
		}
		else{
			veg_vec[veg_vec_ind].wrapp_mobj_i = loader->LoadMeshObjectFactory(path);
			veg_vec[veg_vec_ind].wrapp_mobj_ii = loader->LoadMeshObjectFactory(path_i);
			veg_vec[veg_vec_ind].mobj_i_name = name;
			veg_vec[veg_vec_ind].mobj_ii_name = name_i;
		}
	}
	else{
		veg_vec[veg_vec_ind].wrapp_bill = loader->LoadMeshObjectFactory(path);
		veg_vec[veg_vec_ind].wrapp_mobj_i = loader->LoadMeshObjectFactory(path_i);
		veg_vec[veg_vec_ind].wrapp_mobj_ii = loader->LoadMeshObjectFactory(path_ii);
		veg_vec[veg_vec_ind].bill_name = name;
		veg_vec[veg_vec_ind].mobj_i_name = name_i;
		veg_vec[veg_vec_ind].mobj_ii_name = name_ii;
	}
	
	
	veg_vec[veg_vec_ind].ulv = upper_left_vertex;
	veg_vec[veg_vec_ind].lrv = lower_right_vertex;
	veg_vec[veg_vec_ind].nmbos = nmb_of_species;
	veg_vec[veg_vec_ind].md = min_dist;
	veg_vec[veg_vec_ind].blending = blend;
	veg_vec[veg_vec_ind].rotation = rotation;

	if(blend){
		if((max_bl_dist == 0.0f) && (min_bl_dist == 0.0f)){
		//DEFAULT BLENDING VALUES
			veg_vec[veg_vec_ind].max_blend_dist = MAX_BLEND_DIST;
			veg_vec[veg_vec_ind].min_blend_dist = MIN_BLEND_DIST;
			printf(" NULL: MAX_BLEND_DIST: %f\n",veg_vec[veg_vec_ind].max_blend_dist);
		}
		else{	
			veg_vec[veg_vec_ind].max_blend_dist = max_bl_dist;
			veg_vec[veg_vec_ind].min_blend_dist = min_bl_dist;
			printf(" NOT NULL: MAX_BLEND_DIST: %f\n",veg_vec[veg_vec_ind].max_blend_dist);
		}
		printf("MAX_BLEND_DIST: %f\n",veg_vec[veg_vec_ind].max_blend_dist);
	}
	if(rotation){
		if(rot_dist == 0.0f)
			veg_vec[veg_vec_ind].rot_dist = ROT_DIST;
		else
			veg_vec[veg_vec_ind].rot_dist = rot_dist;
	}
		

	veg_vec_ind++;
	
}

/** 
Only place one object in the world, not loaded in to the world with the Load_objects method.
*/
void csVegFuncSystem::Load_object(boolean blend, float max_bl_dist, float min_bl_dist, boolean rotation, float rot_dist,
								 float x, float z,char* name, char* path, char* name_i = NULL, char* path_i = NULL,
								 char* name_ii = NULL, char* path_ii = NULL){
	if(path_ii == NULL){
		if(path_i == NULL){
			veg_vec[veg_vec_ind].wrapp_bill = loader->LoadMeshObjectFactory(path);
			veg_vec[veg_vec_ind].bill_name = name;
		}
		else{
			veg_vec[veg_vec_ind].wrapp_mobj_i = loader->LoadMeshObjectFactory(path);
			veg_vec[veg_vec_ind].wrapp_mobj_ii = loader->LoadMeshObjectFactory(path_i);
			veg_vec[veg_vec_ind].mobj_i_name = name;
			veg_vec[veg_vec_ind].mobj_ii_name = name_i;
		}
	}
	else{
		veg_vec[veg_vec_ind].wrapp_bill = loader->LoadMeshObjectFactory(path);
		veg_vec[veg_vec_ind].wrapp_mobj_i = loader->LoadMeshObjectFactory(path_i);
		veg_vec[veg_vec_ind].wrapp_mobj_ii = loader->LoadMeshObjectFactory(path_ii);
		veg_vec[veg_vec_ind].bill_name = name;
		veg_vec[veg_vec_ind].mobj_i_name = name_i;
		veg_vec[veg_vec_ind].mobj_ii_name = name_ii;
	}

	veg_vec[veg_vec_ind].x = x;
	veg_vec[veg_vec_ind].z = z;
	veg_vec[veg_vec_ind].blending = blend;
	veg_vec[veg_vec_ind].rotation = rotation;

	if(blend){
		if((max_bl_dist == 0.0f) && (min_bl_dist == 0.0f)){
		//DEFAULT BLENDING VALUES
			veg_vec[veg_vec_ind].max_blend_dist = MAX_BLEND_DIST;
			veg_vec[veg_vec_ind].min_blend_dist = MIN_BLEND_DIST;
			printf(" NULL: MAX_BLEND_DIST: %f\n",veg_vec[veg_vec_ind].max_blend_dist);
		}
		else if((max_bl_dist == 1.0f) && (min_bl_dist == 1.0f)){
			veg_vec[veg_vec_ind].max_blend_dist = MAX_BLEND_DIST;
			veg_vec[veg_vec_ind].min_blend_dist = MIN_BLEND_DIST;
			veg_vec[veg_vec_ind].blending = FALSE;
			veg_vec[veg_vec_ind].switching = TRUE;
		}
		else{	
			veg_vec[veg_vec_ind].max_blend_dist = max_bl_dist;
			veg_vec[veg_vec_ind].min_blend_dist = min_bl_dist;
			printf(" NOT NULL: MAX_BLEND_DIST: %f\n",veg_vec[veg_vec_ind].max_blend_dist);
		}
		printf("MAX_BLEND_DIST: %f\n",veg_vec[veg_vec_ind].max_blend_dist);
	}

	if(rotation){
		if(rot_dist == 0.0f)
			veg_vec[veg_vec_ind].rot_dist = ROT_DIST;
		else{
			veg_vec[veg_vec_ind].rot_dist = rot_dist;
			printf("ROT_DIST: %f\n",veg_vec[veg_vec_ind].rot_dist);
		}
	}
		
	veg_vec_ind++;

}


void csVegFuncSystem::Place_Object(float x, float z, char* path_i, char* path_ii = NULL, char* path_iii = NULL ){
	
	float y = 0.0f;
	float x_tmp = 0.0f;
	float z_tmp = 0.0f;
	
	int x_res = terr_state->GetXResolution();
	int y_res = terr_state->GetYResolution();
	csVector3 top_left = terr_state->GetTopLeftCorner();			
	csVector3 scale = terr_state->GetScale();
	
	x_tmp = x;
	
	x_tmp = x_tmp - (top_left.x);
	z_tmp = z - (top_left.z);
	
	x_tmp = x_tmp * (1/((scale.x) * x_res));
	z_tmp = z_tmp * (1/((scale.z) * y_res));
	
	y = height->GetHeight(x_tmp,z_tmp);
	
	y = y + top_left.y - HEIGHT_OFFSET;
	
	if(path_i != NULL){		
		iMeshFactoryWrapper* meshfact_i = loader->LoadMeshObjectFactory(path_i);
		add_sprite(view, room, meshfact_i, x, y, z);
	}
	if(path_ii != NULL){
		iMeshFactoryWrapper* meshfact_ii = loader->LoadMeshObjectFactory(path_ii);
		add_sprite(view, room, meshfact_ii, x, y, z);
	}
	if(path_iii != NULL){
		iMeshFactoryWrapper* meshfact_iii = loader->LoadMeshObjectFactory(path_iii);
		add_sprite(view, room, meshfact_iii, x, y, z);
	}
	
}

/**
Place a number of objects that have been loaded with the Load_objects method
*/


void csVegFuncSystem::Place_Objects(){
			
	float y = 0.0f;
	float x_tmp = 0.0f;
	float z_tmp = 0.0f;
	float x = 0.0f;
	float z = 0.0f;
	float r = 0.0f;
	
	int x_res = terr_state->GetXResolution();
	int y_res = terr_state->GetYResolution();
	csVector3 top_left = terr_state->GetTopLeftCorner();			
	csVector3 scale = terr_state->GetScale();
	
	
	
	
	for(int i=0;i < veg_vec_ind; i++){
		
		if(veg_vec[i].nmbos != 1){
		
			random_data* veg_data = Random(veg_vec[i].ulv,veg_vec[i].lrv,veg_vec[i].nmbos,veg_vec[i].md);
		
			//printf("BACK FROM RANDOM !!!\n");

			veg_vec[i].nmbos = veg_data[0].nmbofs;
			veg_vec[i].names = new name_data[veg_vec[i].nmbos];
		
			for(int k=0; k < veg_vec[i].nmbos;k++){
				veg_vec[i].names[k].bill_alpha = 0.0f;
				veg_vec[i].names[k].poly_alpha = 0.0f;
			}
		
		
			for (int j=0;j < veg_vec[i].nmbos;j++){
			
				x = 0.0f;
				y = 0.0f;
				z = 0.0f;
			
				x = veg_data[j].x;
				z = veg_data[j].z;
				r = veg_data[j].alpha;
			
			
				//printf("X: %f, Z: %f\n",x,z);
			
				x_tmp = x;
			
				x_tmp = x_tmp - (top_left.x);
				z_tmp = z - (top_left.z);
			
				x_tmp = x_tmp * (1/((scale.x) * x_res));
				z_tmp = z_tmp * (1/((scale.z) * y_res));
			
				y = height->GetHeight(x_tmp,z_tmp);
			
				y = y + top_left.y - HEIGHT_OFFSET;	
			
			
			
				if(veg_vec[i].wrapp_bill != NULL){
					veg_vec[i].names[j].bill_name = new csString(veg_vec[i].bill_name);
					veg_vec[i].names[j].bill_name->Append(j);
					const char* chr = veg_vec[i].names[j].bill_name->GetData();
					if(veg_vec[i].blending)
						add_sprite(view, room, veg_vec[i].wrapp_bill, x, y, z,chr,FALSE, r);
					else
						add_sprite(view, room, veg_vec[i].wrapp_bill, x, y, z,chr,TRUE, r);
				}
				if(veg_vec[i].wrapp_mobj_i != NULL){
					veg_vec[i].names[j].mobj_i_name = new csString(veg_vec[i].mobj_i_name);
					veg_vec[i].names[j].mobj_i_name->Append(j);
					const char* chr = veg_vec[i].names[j].mobj_i_name->GetData();
					if(veg_vec[i].blending)
						add_sprite(view, room, veg_vec[i].wrapp_mobj_i, x, y, z,chr,FALSE, r);
					else
						add_sprite(view, room, veg_vec[i].wrapp_mobj_i, x, y, z,chr,TRUE, r);
				}
				if(veg_vec[i].wrapp_mobj_ii != NULL){
					veg_vec[i].names[j].mobj_ii_name = new csString(veg_vec[i].mobj_ii_name);
					veg_vec[i].names[j].mobj_ii_name->Append(j);
					const char* chr = veg_vec[i].names[j].mobj_ii_name->GetData();
					if(veg_vec[i].blending)
						add_sprite(view, room, veg_vec[i].wrapp_mobj_ii, x, y, z,chr,FALSE, r);
					else
						add_sprite(view, room, veg_vec[i].wrapp_mobj_ii, x, y, z,chr,TRUE, r);
				}
				
			}
			
			delete [] veg_data;
			
		}
		else
		{
			veg_vec[i].names = new name_data[veg_vec[i].nmbos+1];

			veg_vec[i].names[0].bill_alpha = 0.0f;
			veg_vec[i].names[0].poly_alpha = 0.0f;

			float y = 0.0f;
			float x_tmp = 0.0f;
			float z_tmp = 0.0f;
	
			x = veg_vec[i].x;
			z = veg_vec[i].z;


			int x_res = terr_state->GetXResolution();
			int y_res = terr_state->GetYResolution();
			csVector3 top_left = terr_state->GetTopLeftCorner();			
			csVector3 scale = terr_state->GetScale();
	
			x_tmp = x;
	
			x_tmp = x_tmp - (top_left.x);
			z_tmp = z - (top_left.z);
	
			x_tmp = x_tmp * (1/((scale.x) * x_res));
			z_tmp = z_tmp * (1/((scale.z) * y_res));
	
			y = height->GetHeight(x_tmp,z_tmp);
	
			y = y + top_left.y - HEIGHT_OFFSET;
	
			if(veg_vec[i].wrapp_bill != NULL){
				veg_vec[i].names[0].bill_name = new csString(veg_vec[i].bill_name);
				veg_vec[i].names[0].bill_name->Append(0);
				const char* chr = veg_vec[i].names[0].bill_name->GetData();
				if(veg_vec[i].blending)
					add_sprite(view, room, veg_vec[i].wrapp_bill, x, y, z,chr,FALSE, r);
				else
					add_sprite(view, room, veg_vec[i].wrapp_bill, x, y, z,chr,TRUE, r);
			}
			if(veg_vec[i].wrapp_mobj_i != NULL){
				veg_vec[i].names[0].mobj_i_name = new csString(veg_vec[i].mobj_i_name);
				veg_vec[i].names[0].mobj_i_name->Append(0);
				const char* chr = veg_vec[i].names[0].mobj_i_name->GetData();
				if(veg_vec[i].blending)
					add_sprite(view, room, veg_vec[i].wrapp_mobj_i, x, y, z,chr,FALSE, r);
				else
					add_sprite(view, room, veg_vec[i].wrapp_mobj_i, x, y, z,chr,TRUE, r);
			}
			if(veg_vec[i].wrapp_mobj_ii != NULL){
				veg_vec[i].names[0].mobj_ii_name = new csString(veg_vec[i].mobj_ii_name);
				veg_vec[i].names[0].mobj_ii_name->Append(0);
				const char* chr = veg_vec[i].names[0].mobj_ii_name->GetData();
				if(veg_vec[i].blending)
					add_sprite(view, room, veg_vec[i].wrapp_mobj_ii, x, y, z,chr,FALSE, r);
				else
					add_sprite(view, room, veg_vec[i].wrapp_mobj_ii, x, y, z,chr,TRUE, r);
			}
				
		}
			
			
	}
	
	
}





// Create the sprite and add it to the engine.
//MessageBox(NULL,"Before sprite","DEBUG",MB_OK);
//char buf[1024];
//sprintf(buf,"view %d. GetEngine() %d. imeshfact %d. room %d.",view, view->GetEngine(), imeshfact, room);
//MessageBox(NULL,buf,"DEBUG",MB_OK);

//MessageBox(NULL,"After sprite","DEBUG",MB_OK);
//printf("NOT ENGINE\n");
