/*
    Copyright (C) 2002 by Erik nihlmar and Mikael Rosberg

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _VEGFUNC_H_
#define _VEGFUNC_H_

#include "ivaria/vegfunc.h"
#include "iutil/comp.h"
#include "iengine/engine.h"
#include "csutil/csstring.h"

struct iLoader;
struct iGraphics3D;
struct iView;

#define NUMBER_OF_OBJECTS 100

/**
 * Implementation of the vegetation planter system.
 */
class csVegFuncSystem : public iVegFuncSystem
{
private:
	iSector* room;
	iLoader* loader;
	iGraphics3D* g3d;
	iView* view;
	iTerrainHeightFunction* height;
	iTerrFuncState* terr_state;


	
	//float poly_alpha;
	//float bill_alpha;

	struct name_data{
		csString* bill_name;
		csString* mobj_i_name;
		csString* mobj_ii_name;
		float bill_alpha;
		float poly_alpha;
	};

	struct obj_struct{
		iMeshFactoryWrapper* wrapp_bill;
		iMeshFactoryWrapper* wrapp_mobj_i;
		iMeshFactoryWrapper* wrapp_mobj_ii;
		char* bill_name;
		char* mobj_i_name;
		char* mobj_ii_name;
		csVector2 *ulv;
		csVector2 *lrv;
		csVector3 *origin;
		int nmbos;
		float md;
		boolean blending;
		boolean rotation;
		//-----HACK--------------
		boolean switching;
		//-----------------------
		name_data* names;
		float max_blend_dist;
		float min_blend_dist;
		float rot_dist;
		float x;
		float z;
	};

	

 
	//-----------------------------------------
	obj_struct veg_vec[NUMBER_OF_OBJECTS];
	int veg_vec_ind;
	//-----------------------------------------


public:
  SCF_DECLARE_IBASE;

  /// Create the plugin object
  csVegFuncSystem (iBase *pParent);
  ~csVegFuncSystem();
  /// Checking if we have contact
  virtual void Contact();
  virtual void Init_VegFunc(iLoader* loader, iGraphics3D* g3d, iView* view);
  virtual void Init_VegFunc_Height(iTerrainHeightFunction* height, iTerrFuncState* terr_state, iSector* room);
  virtual void Load_objects(boolean blend,float max_bl_dist, float min_bl_dist,boolean rotation,float rot_dist, 
						csVector2* upper_left_vertex, csVector2* lower_right_vertex, int nmb_of_species, 
						   float min_dist,char* name,char* path,char* name_i, char* path_i,
						   char* name_ii, char* path_ii);
  virtual void Load_object(boolean blend, float max_bl_dist, float min_bl_dist, boolean rotation, float rot_dist,
								 float x, float z,char* name, char* path, char* name_i, char* path_i,
								 char* name_ii, char* path_ii);

  virtual void Place_Object(float x, float z, char* path_i, char* path_ii,char* path_iii);
  virtual void Place_Objects();
  virtual void Billboard_Rotation();
  virtual void Blending();
  struct eiComponent : public iComponent
  {
    SCF_DECLARE_EMBEDDED_IBASE(csVegFuncSystem);
    virtual bool Initialize (iObjectRegistry*) { return true; }
  } scfiComponent;

};

#endif // _VEGFUNC_H_



